@@ -12,8 +12,10 @@ SF_RESPONSE_CHUNK_SIZE = 4 * 1024
 
 LOGGER_CONFIG_FILENAME = "logger.ini"
 
-SF_FTP_SERVER = "upload.sourceforge.net"
-SF_FTP_DIRECTORY = "incoming"
+SF_SFTP_SERVER = "frs.sourceforge.net"
+SF_SFTP_PORT = 22
+SF_SFTP_DIRECTORY = "uploads"
+
 
 SF_WARNING = """
 The follwing files could not be added to your release
