@@ -22,4 +22,4 @@ class MessageQueue(Queue):
 mainMessageQueue = MessageQueue()
 progressMessageQueue = MessageQueue()
 freshmeatMessageQueue = MessageQueue()
-ftpProgressQueue = MessageQueue()
+sftpProgressQueue = MessageQueue()
