@@ -6,7 +6,7 @@ import logging
 import os
 from ReleaseDataVO import ReleaseDataVO
 from FileVO import FileVO
-from messageQueue import progressMessageQueue, ftpProgressQueue
+from messageQueue import progressMessageQueue, sftpProgressQueue
 from workerThread import WorkerThread
 from help import Help
 import time
@@ -79,9 +79,9 @@ class ReleaseWizard(ReleaseWizardBA):
         self.connect(self.timer, SIGNAL("timeout()"),
                      self.getProgressMsg)
 
-        self.ftp_timer = QTimer(self)
-        self.connect(self.ftp_timer, SIGNAL("timeout()"),
-                     self.update_ftp_progress)
+        self.sftp_timer = QTimer(self)
+        self.connect(self.sftp_timer, SIGNAL("timeout()"),
+                     self.update_sftp_progress)
         
         self.connect(self, SIGNAL("selected(const QString &)"), self.pageChanged)
         self.releaseNameEdited(self.newReleaseNameLineEdit.text())
@@ -200,12 +200,12 @@ class ReleaseWizard(ReleaseWizardBA):
     def show_file_progress(self, show):
         debug("show_file_progress: %s", show)
         if show:
-            self.ftp_timer.start(50, False)
+            self.sftp_timer.start(50, False)
             self.fileTextLabel.show()
             self.fileProgressBar.setProgress(-1)
             self.fileProgressBar.show()
         else:
-            self.ftp_timer.stop()
+            self.sftp_timer.stop()
             self.fileTextLabel.hide()
             self.fileProgressBar.hide()
 
@@ -664,7 +664,7 @@ class ReleaseWizard(ReleaseWizardBA):
         release_notes = data_tuple[0]
         change_log = data_tuple[1]
         files = data_tuple[2]
-              
+
         self.releaseNotesTextEdit.setText(from_utf8(release_notes))
         self.changeLogTextEdit.setText(from_utf8(change_log))
         self.setNextEnabled(self.page(WIZARD_PAGE_RELEASE_NAME), False)
@@ -763,13 +763,19 @@ class ReleaseWizard(ReleaseWizardBA):
         # populate the progress label with it.
         msg = progressMessageQueue.get()
         if msg:
-            self.progressTextLabel.setText(msg)            
-            self.incrementProgressBar()
+            self.progressTextLabel.setText(msg)
+            if not msg.startswith("Retry: "):
+                self.incrementProgressBar()
+#            else:
+#                self.decrementProgressBar(1)
                 
 
     def incrementProgressBar(self, increment=1):
         self.progressBar.setProgress(self.progressBar.progress() + increment)
 
+    def decrementProgressBar(self, decrement=1):
+        self.progressBar.setProgress(self.progressBar.progress() - decrement)
+
 
     def isCancelled(self):
         return self.cancelled
@@ -1054,7 +1060,7 @@ class ReleaseWizard(ReleaseWizardBA):
 
     def error(self, msg):
         self.timer.stop()
-        self.ftp_timer.stop()
+        self.sftp_timer.stop()
         page = self.page(WIZARD_PAGE_PROGRESS)
         self.progressTextLabel.setText(msg)
         self.setFinishEnabled(page, False)
@@ -1065,7 +1071,7 @@ class ReleaseWizard(ReleaseWizardBA):
 
     def complete(self, data):
         self.timer.stop()
-        self.ftp_timer.stop()
+        self.sftp_timer.stop()
         page = self.page(WIZARD_PAGE_PROGRESS)
         if self.create:
             self.progressTextLabel.setText("Successfully created new release")
@@ -1107,10 +1113,10 @@ class ReleaseWizard(ReleaseWizardBA):
         self.notifyTextLabel.setEnabled(False)
 
 
-    def update_ftp_progress(self):
+    def update_sftp_progress(self):
         # read the message queue.  If an item exists,
         # populate the progress label with it.
-        progress_tuple = ftpProgressQueue.get()
+        progress_tuple = sftpProgressQueue.get()
         #debug("update_ftp_progress: %s",  progress_tuple)
         if progress_tuple:
             if progress_tuple == (True, True):
