@@ -1,7 +1,7 @@
 from workerThread import WorkerThread
 from sfurls import *
 from sfregex import *
-from constants import SF_COOKIE_FILENAME, SF_FTP_SERVER, SF_FTP_DIRECTORY
+from constants import SF_COOKIE_FILENAME, SF_SFTP_SERVER, SF_SFTP_DIRECTORY, SF_SFTP_PORT
 
 from ProjectVO import ProjectVO
 from PackageVO import PackageVO
@@ -12,14 +12,19 @@ try:
 except:
     # python2.3 compatibility
     from python24.urllib2 import build_opener, HTTPCookieProcessor, ProxyHandler
+import MultipartPostHandler
 import logging
-from messageQueue import mainMessageQueue, progressMessageQueue, ftpProgressQueue
+from messageQueue import mainMessageQueue, progressMessageQueue, sftpProgressQueue
 import os, sys
-from ftp import FTP_
+import paramiko
 import time
 from constants import SF_RESPONSE_CHUNK_SIZE
+from types import StringType
+import tempfile
 
 debug = logging.getLogger("sfcomm").debug
+info = logging.getLogger("sfcomm").info
+warn =  logging.getLogger("sfcomm").warn
 error = logging.getLogger("sfcomm").error
 exception = logging.getLogger("sfcomm").exception
 
@@ -43,6 +48,18 @@ except:
 
 TRUNCATE_POST = 1024
 
+class RetryException(Exception): pass
+
+def unescape_entities(s):
+    # replace &amp;  &lt;  &gt;  &quot;  in s with their text counterparts
+    if not s: return s
+    s = s.replace("&amp;", "&")
+    s = s.replace("&lt;", "<")
+    s = s.replace("&gt;", ">")
+    s = s.replace("&quot;", '"')
+    return s
+    
+
 class SFComm:
     def __init__(self, data_path, proxy=None):
         self.cookie_file = os.path.join(data_path, SF_COOKIE_FILENAME)
@@ -64,15 +81,17 @@ class SFComm:
                   queue=progressMessageQueue,
                   addl_headers=None,
                   ok_status=None,
+                  post_as_multipart=None,
+                  retry=True,
                   dbg=None):
         #
         # parameters:
         # url: url to retrieve
-        # post_encoded: the encoded query string sent to SF
+        # post_encoded: the (preferably) encoded query string sent to SF
         # queue: the python Queue that status information will be sent to
         # addl_headers: any additional headers that need to be passed to SF
         # ok_status: a string as described below.
-        #
+        # post_as_multipart: post form data with enctype: multipart/form-data
         # returns a tuple containing the data returned from SF and the status.
         #
         # If ok_status is provided, the status returned is True if the
@@ -81,26 +100,59 @@ class SFComm:
         #
         # If ok_status is None, status is always True and data will not be truncated
         #
+        # If retry is True, then this is the first attempt and if it fails this transaction
+        # will be retried.
+        #
         # return tuple form: (data, status)
         #
         data = ""
         ok = True
         try:
             debug("Fetching: %s", url)
-            if post_encoded:
+            if post_encoded and type(post_encoded) is StringType:
                 if len(post_encoded) > TRUNCATE_POST: truncated = "..."
                 else: truncated = ""
+                
                 debug("posted: %s%s", post_encoded[:TRUNCATE_POST], truncated)
 
-            queue.put("Fetching: %s" % url)
+            if not retry:
+                # has already been tried, this is the retry attempt
+                queue.put("Retry: %s" % url)
+            else:
+                # has not be attempted yet... will retry if this fails
+                queue.put("Fetching: %s" % url)
             
             cj = cookielib.MozillaCookieJar()
             cj.load(self.cookie_file, True)
-            opener = build_opener(HTTPCookieProcessor(cj))
+            if post_as_multipart:
+                # enctype: multipart/form-data
+                opener = build_opener(HTTPCookieProcessor(cj),
+                                      MultipartPostHandler.MultipartPostHandler)
+            else:
+                opener = build_opener(HTTPCookieProcessor(cj))
+
             if addl_headers:
                 opener.addheaders = addl_headers
                 
-            r = opener.open(url, post_encoded)
+            try:
+                r = opener.open(url, post_encoded)
+            except Exception, e:
+                exception(e)
+                error("failed posting: %s",  post_encoded)
+                if retry:
+                    error("retrying last action")
+                    return self.fetch_url(url,
+                                          post_encoded,
+                                          queue,
+                                          addl_headers,
+                                          ok_status,
+                                          post_as_multipart,
+                                          False,
+                                          dbg)
+                else:
+                    error("giving up")
+                    raise RetryException("Retry failed... giving up")
+                
 #            debug("r methods: %s", dir(r))
 #            cj.save(self.cookie_file, True)
             if not ok_status: 
@@ -129,8 +181,10 @@ class SFComm:
             r.close()
             if dbg: debug("Done")
             #debug("length: %ld  -  url: %s", len(data), url)
+        except RetryException, e:
+            raise
         except Exception, e:
-            print e
+            exception(e)
 
         if dbg: debug("DATA: %s", data)
         return data, ok
@@ -140,6 +194,8 @@ class SFComm:
     def login(self, username, password, return_to=None):
         # returns a set of projects for the given username/password
         # or None
+        self.username = username  # used by sftp
+        self.password = password  # used by sftp
         debug("Attempting to login to SourceForge")
         form = {'form_loginname': username,
                 'form_pw': password,
@@ -168,13 +224,14 @@ class SFComm:
 
         cj.save(self.cookie_file, True, True) # ignore discard! ignore expires
         
+        projects = set()
+
         data = r.read()
         idx = data.find("Invalid Password or User Name")
         if idx != -1:
             debug("Could not login")
             return projects
 
-        projects = set()
         #debug(data)
         project_tuples = PROJECTS_FROM_MY_PROJECTS.findall(data)
         for pt in project_tuples:
@@ -186,7 +243,6 @@ class SFComm:
         return projects
 
 
-
     def get_packages(self, group_id):
         url = "%s=%s" % (EDIT_PACKAGE_BASE_URL, group_id)
         packages = set()
@@ -237,32 +293,7 @@ class SFComm:
             mainMessageQueue.put("You do not have the proper permissions")
             return False
         else: return True
-        
 
-    def add_release(self, group_id, package_id, name):
-        url = "%s?package_id=%s&group_id=%s" % (NEW_RELEASE_URL,
-                                                package_id,
-                                                group_id)
-        form = {'package_id': package_id,
-                'group_id': group_id,
-                'release_name': name,
-                'newrelease': 'yes',
-                'submit': 'Create This Release'}
-
-        data = self.fetch_url(url, urlencode(form))[0]
-        #print url
-        m = GET_RELEASE_ID.search(data)
-        if m:
-            return m.group("releaseid")
-        else:
-            if data.find("Error creating Project object") != -1:
-                progressMessageQueue.put("Error creating Project object")
-            elif data.find("Login to SourceForge.net") != -1:
-                progressMessageQueue.put("Login required?")
-                #print data
-            else:
-                progressMessageQueue.put("Error creating new release")
-            return None
 
     def get_releases(self, group_id, package_id):
         url = "%s?group_id=%s&package_id=%s" % (EDIT_RELEASE_URL,
@@ -293,8 +324,8 @@ class SFComm:
         
         m = GET_NOTES_AND_LOG.search(data)        
         if m:
-            release_notes = m.group("releasenotes")
-            change_log = m.group("changelog")
+            release_notes = unescape_entities(m.group("releasenotes"))
+            change_log = unescape_entities(m.group("changelog"))
 
         filedata = GET_FILE_NAMES_ETC.findall(data)
         files = []
@@ -313,6 +344,7 @@ class SFComm:
             debug("file: %s - %s - %s - %s", fileid, filename, processor, filetype)
         
         debug("edit release - files: %s", files)
+    
         return (release_notes, change_log, files)
 
 
@@ -338,8 +370,6 @@ class SFComm:
         return result
     
 
-
-
     def add_release(self, group_id, package_id, name):
         url = "%s?package_id=%s&group_id=%s" % (NEW_RELEASE_URL,
                                                 package_id,
@@ -363,6 +393,7 @@ class SFComm:
                 #print data
             else:
                 progressMessageQueue.put("Error creating new release")
+                error("error creating new release (could not find releaseid): " + data)
             return None
 
 
@@ -383,19 +414,92 @@ class SFComm:
                 'status_id': status_id,
                 'uploaded_notes': '',
                 'uploaded_changes': '',
-                'release_notes': release_notes,
+                'release_notes': '', 
+                'release_changes': '',
+                'preformatted': '1',
+                'submit': 'Submit/Refresh'
+                }
+
+        # attempt to create temp files to store the release notes and change log
+        # if unable to do so or if it cannot completed the task fallback to old behavior
+        # and simply post the text of each as-is.
+        try:
+            release_notes_fp, release_notes_path = tempfile.mkstemp(prefix="rf-", text=True)
+            os.write(release_notes_fp, "%s\n" % release_notes)
+            os.close(release_notes_fp)
+            form['uploaded_notes'] = open(release_notes_path, "r")
+        except:
+            warn("could not create tempfile for release_notes, reverting to old implementation")
+            release_notes_path = ""
+            
+        try:
+            change_log_fp, change_log_path = tempfile.mkstemp(prefix="rf-", text=True)
+            os.write(change_log_fp, "%s\n" % change_log)
+            os.close(change_log_fp)
+            form['uploaded_changes'] = open(change_log_path, "r")        
+        except:
+            warn("could not create tempfile for change_log, reverting to old implementation")
+            change_log_path = ""
+            
+        if not change_log_path or not release_notes_path:
+            return self.edit_release_step1_fallback(group_id, package_id, release_id,
+                                                    name, release_notes, change_log, status)
+       
+        try:
+            ok = self.fetch_url(url,
+                                form, # encoded by MultipartPostHandler
+                                post_as_multipart=True,
+                                ok_status="Data Saved")[1]  
+        except:
+            ok = self.edit_release_step1_fallback(group_id, package_id, release_id,
+                                                  name, release_notes, change_log, status)
+        
+        # if the temp files were created, remove them now       
+        if release_notes_path:
+            try: os.remove(release_notes_path)
+            except: pass
+
+        if change_log_path:
+            try: os.remove(change_log_path)
+            except: pass
+            
+        return ok
+
+
+    def edit_release_step1_fallback(self, group_id, package_id, release_id,
+                                    name, release_notes, change_log, status='active'):        
+        url = EDIT_RELEASE_URL
+        if status.lower() == 'active': status_id = '1'
+        else: status_id = '3'
+
+        warn("using (post) fallback method... html entities will be munged")
+        
+        form = {'package_id': package_id,
+                'new_package_id': package_id,
+                'group_id': group_id,
+                'release_id': release_id,
+                'release_name': name,
+                'release_date': time.strftime("%Y-%m-%d"),
+                'step1': '1',
+                'status_id': status_id,
+                'uploaded_notes': '',
+                'uploaded_changes': '',
+                'release_notes': release_notes, 
                 'release_changes': change_log,
                 'preformatted': '1',
                 'submit': 'Submit/Refresh'
                 }
-        ok = self.fetch_url(url,
-                            urlencode(form),
-                            ok_status="Data Saved")[1]
+
+        try:
+            ok = self.fetch_url(url,
+                                urlencode(form), 
+                                ok_status="Data Saved",
+                                retry=False)[1]
+        except:
+            ok = False
+        
         return ok
-        #if data.find("Data Saved") != -1: return True
-        #else: return False
-
-
+    
 
     def edit_release_step2(self, group_id, package_id, release_id, fileVOs):
         url = EDIT_RELEASE_URL
@@ -421,7 +525,7 @@ class SFComm:
             error("Error selecting %s for inclusion", filename)
             #debug("dumping data: %s", data)  # remove me...
             ok = False
-        
+
         matches = GET_FILE_ID.findall(data)
         lookup = {}
         for match in matches:
@@ -432,7 +536,7 @@ class SFComm:
 
     def edit_release_step3(self, group_id, package_id, release_id, fileVOs, lookup_dict):
         url = EDIT_RELEASE_URL
-        
+
         form = {'package_id': package_id,
                 'group_id': group_id,
                 'release_id': release_id,
@@ -446,8 +550,10 @@ class SFComm:
             form['processor_id'] = fileVO.getProcessorId()
             form['type_id'] = fileVO.getFileTypeId()
             try:
-                #debug("filename: %s", fileVO.getFilename() )
-                #debug("dict: %s", str(lookup_dict))
+                # REMOVE THESE!!!!! for debugging purpose re: twb
+                #error("filename: %s", fileVO.getFilename() )
+                #error("dict: %s", str(lookup_dict))
+                ######
                 
                 form['file_id'] = lookup_dict[fileVO.getFilename()]
                 ok &= self.fetch_url(url, urlencode(form), ok_status="File Updated")[1]
@@ -510,29 +616,23 @@ class SFComm:
                                        status)
 
 
+
     def upload_files(self, parent, fileVOs):
-        ftp = FTP_()
-
         try:
-            ftp.connect(SF_FTP_SERVER)
-            ftp.getwelcome()
-
-            ftp.login("ftp", "releaseforge")
-            ftp.cwd(SF_FTP_DIRECTORY)
+            t = paramiko.Transport((SF_SFTP_SERVER, SF_SFTP_PORT))
+            t.start_client()  
+            t.auth_password(self.username, self.password)
+            sftp = paramiko.SFTPClient.from_transport(t)
+            sftp.chdir(SF_SFTP_DIRECTORY)
+            
             num_uploaded = 0
-
             for fileVO in fileVOs:
                 fullpath = fileVO.getFullPath()
                 filename = fileVO.getFilename()
                 debug("uploading %s (%s)", filename, fullpath)
                 progressMessageQueue.put("Uploading: %s" % filename)
-                f = open(fullpath, "rb")
-                
-                ftp.storbinary('STOR ' + filename,
-                               f,
-                               callback=self.update_ftp_progress)
-                f.close()
 
+                sftp.put(fullpath, filename, callback=self.update_sftp_progress)
                 num_uploaded += 1
                 if parent.isCancelled(): break
 
@@ -542,21 +642,22 @@ class SFComm:
                 msg = "Successfully uploaded %d files" % num_uploaded
         except Exception, e:
             msg = str(e)
-            progressMessageQueue.put("FTP error: %s" % msg)
+            error(msg)
+            progressMessageQueue.put("SFTP error: %s" % msg)
         debug(msg)
         try:
-            ftp.close()
+            sftp.close()
         except: pass
 
-        ftpProgressQueue.clear()
-        ftpProgressQueue.put( (True, True) )
+        sftpProgressQueue.clear()
+        sftpProgressQueue.put( (True, True) )
         return msg
 
 
-    def update_ftp_progress(self, progress):
+    def update_sftp_progress(self, progress, total):
         #debug("progress: %s", progress)
-        ftpProgressQueue.clear()
-        ftpProgressQueue.put(progress)
+        sftpProgressQueue.clear()
+        sftpProgressQueue.put( (progress, total) )
 
 
     def submit_news(self, group_id, subject, message):
